const snekfetch = require("snekfetch");
const endpoint = "https://discordapp.com/api/v6/webhooks";

class Webhook {
    
    
    constructor(url) {
        this.url = url;
        this.id = "";
        this.token = ""; 
        this.meta = {};
        this.ready = false;
        snekfetch.get(this.url)
            .then((res) => {
                let parsed = JSON.parse(res.text);
                Object.assign(this.meta, parsed);
                this.id = parsed.id;
                this.token = parsed.token;
                this.ready = true;
            })
            .catch((err) => {
                throw err;
            });
    }

    
    
    verifyName(name) {
        if (name.length > 30) {
            throw new Error(`[CHARACTER LIMIT] The Name Specified Was ${name.length} Characters, Discord Webhook Names Must Be Under 32 Characters, However We Reduced The Character Limit`);
        }
    } 
    
    
    sendPayload(payload) {
        this.verifyName(payload.username);
        if (this.ready) {
            return new Promise((resolve, reject) => {
                snekfetch.post(this.endpoint)
                    .send(payload)
                    .then(() => {
                        resolve();
                    })
                    .catch((err) => {
                        let e = JSON.parse(err.text);
                        if (e.retry_after) { 
                            setTimeout(() => {
                                this.sendPayload(payload);
                            }, e.retry_after);
                        } else {
                            reject(err);
                        }
                    });
            });
        } else {
            setTimeout(() => {
                this.sendPayload(payload);
            }, 25);
        }
    }

    
    
    get endpoint() {
        return `${endpoint}/${this.id}/${this.token}/slack`;
    }

    
    
    Err(name, message) {
        let payload = {
            "username": name,
            "attachments": [{
                "color": "#ff0000",
                "fields": [{
                    "title": "Error",
                    "value": message
                }],
                
            }]
        };

        return this.sendPayload(payload);
    }

    
   
    Info(name, message) {
        let payload = {
            "username": name,
            "attachments": [{
                "color": "#00fffa",
                "fields": [{
                    "title": "Information",
                    "value": message
                }],
              
            }]
        };

        return this.sendPayload(payload);
    }
    
    
    Success(name, message) {
        let payload = {
            "username": name,
            "attachments": [{
                "color": "#04ff00",
                "fields": [{
                    "title": "Success",
                    "value": message
                }],
               
            }]
        };

        return this.sendPayload(payload);
    }
    
  
    Warn(name, message) {
        let payload = {
            "username": name,
            "attachments": [{
                "color": "#ffe900",
                "fields": [{
                    "title": "Warning",
                    "value": message
                }],
     
            }]
        };

        return this.sendPayload(payload);
    }

    
    Custom(name, message, title, color, imageUrl) {
        let payload = {
            "username": name,
            "attachments": [{
                "fields": [{
                    "title": title,
                    "value": message
                }],
              
            }]
        };
        if (color) {
            payload.attachments[0].color = color;
        }
        if (imageUrl) {
            payload.attachments[0].image_url = imageUrl;
        }
        return this.sendPayload(payload);
    }

   
    Image(name, message, title, imageUrl) {
        let payload = {
            "username": name,
            "attachments": [{
                "fields": [{
                    "title": title,
                    "value": message
                }],
                "image_url": imageUrl
            }]
        };

        return this.sendPayload(payload);
    }
    
    Announcement(name, message) {
        let payload = {
            "username": name,
            "attachments": [{
                "fields": [{
                    "title": "Announcement",
                    "value": message
                }],
                
            }]
        };

        return this.sendPayload(payload);
    }
}

module.exports = Webhook;
